from earthdiagnostics import Utils
from autosubmit.config.log import Log
import os
import shutil
import traceback
from nco import NCOException


class Heat(object):

    @staticmethod
    def layer(input_file, output_file, upper, lower):
        """
        Pointwise Ocean Heat Content in a specified ocean thickness (J/m-2)

        Created in June 2012      Author : isabel.andreu-burillo@ic3.cat
        May 2014 - Virginie Guemas - Way around the bc that does not work on moore

        :param input_file: input grid_T file nam
        :param output_file: output file name  (=> 2D x-y )
        :param upper: upper depth of the layer (in meters)
        :param lower: lower depth of the layer (in meters)
        :return:
        """

        cdo = Utils.cdo
        nco = Utils.nco

        temp = Utils.get_temp_file()
        temp2 = Utils.get_temp_file()
        heatc_sl_out = Utils.get_temp_file()
        heatc_sl_top = Utils.get_temp_file()
        heatc_sl_bottom = Utils.get_temp_file()
        heatc_sl_invert = Utils.get_temp_file()
        e3tfile = Utils.get_temp_file()

        nco.ncap2(input='mesh_zgr.nc', output=e3tfile, options='-v -O -s "heatc_sl=tmask*e3t"'),
        nco.ncrename(input=e3tfile, output=e3tfile, options='-d t,time -d z,deptht')

        nco.ncks(input=input_file, output=temp, options='-O -v votemper')
        nco.ncrename(input=temp, output=temp, options='-v votemper,heatc_sl')
        cdo.mul(input=' '.join([temp, e3tfile]), output=heatc_sl_out)
        os.remove(e3tfile)
        nco.ncks(input=heatc_sl_out, output=temp, options='-O -d deptht,{0}.0,{1}.0'.format(upper, lower))

        # perform the integration of ohc down to that level (main contribution)
        nco.ncap2(input=temp, output=heatc_sl_top, options='-O -s "heatc_sl=heatc_sl.total($deptht)"')

        # now extract a few levels below, to compute the residual ohc
        # obtain the weight for the extra level containing the 300 m
        # deptht in the gridT files is positive
        # weight = (300.0 - depth_top)/(depth_bottom - depth_top)
        # and add the thickness down to 300 m in the next layer
        nco.ncpdq(input=heatc_sl_out, output=heatc_sl_invert, options='-a "-deptht"')
        nco.ncks(input=heatc_sl_invert, output=temp2, options='-O -d deptht,0,0')
        Utils.rename_variable(temp2, 'deptht', 'layerthcknss')
        shutil.move(temp2, heatc_sl_invert)

        nco.ncks(input=heatc_sl_out, output=heatc_sl_bottom,
                 options='-O -d deptht,{0}.0,{1}.0'.format(lower, lower + 200))
        nco.ncks(input=heatc_sl_bottom, output=heatc_sl_bottom, options='-O -d deptht,0,0')
        Utils.rename_variable(heatc_sl_bottom, 'deptht', 'layerthcknss')

        os.remove(heatc_sl_out)

        # Here, add the residual contribution, before adding it to the main contribution
        nco.ncbo(input=' '.join([heatc_sl_bottom, heatc_sl_invert]), output=temp,
                 options='-A --op_typ=sub -v layerthcknss')

        Utils.rename_variable(temp, 'layerthcknss', 'heatc_sl')
        nco.ncap2(input=heatc_sl_invert, output=temp2, options='-s "heatc_sl=({0:.1f} - layerthcknss)"'.format(lower))
        os.remove(heatc_sl_invert)
        nco.ncbo(input=' '.join([temp, temp2]), output=temp,  options='--op_typ=/ -v heatc_sl')
        os.remove(temp2)
        nco.ncrename(input=temp, output=temp, options='-v heatc_sl,factor')
        nco.ncks(input=temp, output=heatc_sl_bottom, options='-A -v factor')
        nco.ncap2(input=heatc_sl_bottom, output=heatc_sl_bottom, options='-O -s "heatc_sl=(factor * heatc_sl)"')
        nco.ncwa(input=heatc_sl_bottom, output=heatc_sl_bottom, options='-O -a deptht')
        nco.ncbo(input=' '.join([heatc_sl_top, heatc_sl_bottom]), output=temp,  options='--op_typ=+ -v heatc_sl')
        os.remove(heatc_sl_top)
        os.remove(heatc_sl_bottom)
        nco.ncap2(input=temp, output=temp, options='-s "heatc_sl=1020.0*4000*heatc_sl"')

        Utils.setminmax(temp, ['heatc_sl'])

        shutil.move(temp, output_file)
        os.remove(temp)


def main():
    try:
        Heat.layer("ORCA1_MM_19601101_19610228_grid_T.nc", "out.nc", 300, 2000)
    except NCOException as ex:
        Log.error('Exception {0}: stderr: {1}, stdout: {2}',  ex.msg, ex.stderr, ex.stdout)
        Log.error(traceback.format_exc())


if __name__ == "__main__":
    main()
